%defines
%debug
%error-verbose
%define api.pure full
%locations
%parse-param { size_t *nerr }
%lex-param { size_t *nerr }

%code requires
{
#include <stddef.h>
}

%code provides
{
#define YY_DECL                                 \
  enum yytokentype yylex (YYSTYPE *yylval, YYLTYPE *yylloc, size_t *nerr)
  YY_DECL;
}

%code
{
#include <stdio.h>
#include <stdlib.h>

  void yyerror (const YYLTYPE *loc, size_t *nerr, const char* msg);
}

%expect 0

%token
  TOK_STAR "*" TOK_SLASH "/"
  TOK_PLUS "+" TOK_MINUS "-"
  TOK_LPAR "(" TOK_RPAR ")"
  TOK_EOL  "end of line"

%token <ival> TOK_NUMBER "number"
%type <ival> exp
%union
{
  int ival;
}
%printer { fprintf (yyoutput, "%d", $$); } <ival>

%left "+" "-"
%left "*" "/"

%%
input:
        line
| input line
;

line:
  exp   TOK_EOL { printf ("%d\n", $exp); }
|       TOK_EOL
| error TOK_EOL { yyerrok; }
;

exp:
  exp "+" exp   { $$ = $1 + $3; }
| exp "-" exp   { $$ = $1 - $3; }
| exp "*" exp   { $$ = $1 * $3; }
| exp "/" exp 
  {
    if ($3)
      $$ = $1 / $3;
    else
      {
        yyerror (&@3, nerr, "invalid mec du bas");
        YYERROR;
      }
  }
| "(" exp ")"   { $$ = $2; }
| "(" error ")" { $$ = 666; }
| "number"      { $$ = $1; }
;
%%
void
yyerror (const YYLTYPE *loc, size_t *nerr, const char* msg)
{
  ++*nerr;
  YY_LOCATION_PRINT (stderr, *loc);
  fprintf (stderr, ": %s\n", msg);
}

extern int yy_flex_debug;
int
main (int argc, const char *argv[])
{
  (void) argc;
  (void) argv;
  yydebug = !!getenv ("PARSE");
  yy_flex_debug = !!getenv ("SCAN");
  size_t nerr = 0;
  yyparse (&nerr);
  return !!nerr;
}
